use rapier2d::{
	dynamics::{
		CCDSolver, ImpulseJointSet, IntegrationParameters, IslandManager, MultibodyJointSet,
		RigidBodySet,
	},
	geometry::{BroadPhase, ColliderSet, NarrowPhase},
	na::vector,
	pipeline::{EventHandler, PhysicsPipeline},
};

pub struct Physics {
	pub rigid_body_set: RigidBodySet,
	pub collider_set: ColliderSet,

	ip: IntegrationParameters,
	pp: PhysicsPipeline,
	im: IslandManager,
	bp: BroadPhase,
	np: NarrowPhase,
	ijs: ImpulseJointSet,
	mjs: MultibodyJointSet,
	ccd: CCDSolver,
}

impl Physics {
	pub fn new() -> Self {
		Self {
			rigid_body_set: RigidBodySet::new(),
			collider_set: ColliderSet::new(),

			ip: IntegrationParameters::default(),
			pp: PhysicsPipeline::new(),
			im: IslandManager::new(),
			bp: BroadPhase::new(),
			np: NarrowPhase::new(),
			ijs: ImpulseJointSet::new(),
			mjs: MultibodyJointSet::new(),
			ccd: CCDSolver::new(),
		}
	}

	pub fn step(&mut self, t: f32, handler: &dyn EventHandler) {
		self.ip.dt = t;
		self.pp.step(
			&vector![0.0, 0.0],
			&self.ip,
			&mut self.im,
			&mut self.bp,
			&mut self.np,
			&mut self.rigid_body_set,
			&mut self.collider_set,
			&mut self.ijs,
			&mut self.mjs,
			&mut self.ccd,
			None,
			&(),
			handler,
		);
	}
}
